from enum import Enum


class DataType(Enum):
    # these need to match the ones in nema-artifacts-model
    STRING = "STRING"
    INT = "INT"
    FLOAT = "FLOAT"
    BOOL = "BOOL"
    CURRENCY = "CURRENCY"
    PERCENTAGE = "PERCENTAGE"
    INT_WITH_PHYSICAL_UNIT_V0 = "INT_WITH_PHYSICAL_UNIT.V0"
    FLOAT_WITH_PHYSICAL_UNIT_V0 = "FLOAT_WITH_PHYSICAL_UNIT.V0"
    INT_WITH_ARBITRARY_UNIT_V0 = "INT_WITH_ARBITRARY_UNIT.V0"
    FLOAT_WITH_ARBITRARY_UNIT_V0 = "FLOAT_WITH_ARBITRARY_UNIT.V0"
    LAT_LONG_EARTH_COORDINATE_V0 = "LAT_LONG_EARTH_COORDINATE.V0"
    LINE_FIGURE_V0 = "LINE_FIGURE.V0"
    SUNBURST_FIGURE_V0 = "SUNBURST_FIGURE.V0"
    VEGA_FIGURE_V0 = "VEGA_FIGURE.V0"
    CSV_V0 = "CSV.V0"
    IMAGE_V0 = "IMAGE.V0"
    ARBITRARY_FILE_V0 = "ARBITRARY_FILE.V0"
    ARBITRARY_FILE_COLLECTION_V0 = "ARBITRARY_FILE_COLLECTION.V0"
    DATE_V0 = "DATE.V0"
    DATE_TIME_V0 = "DATE_TIME.V0"
